# Copyright (c) 2022, Muhammad Asad (masadcv@gmail.com)
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.

# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import numpy as np
import numpymaxflowcpp


def maxflow(
    image: np.ndarray, 
    prob: np.ndarray, 
    lamda: float, 
    sigma: float, 
    connectivity: int = 0
) -> np.ndarray:
    r"""Computes Max-flow/Min-cut in numpy for 2D images and 3D volumes follwing methods described in:
    
    Boykov, Yuri, and Vladimir Kolmogorov. 
    "An experimental comparison of min-cut/max-flow algorithms for energy minimization in vision." 
    IEEE transactions on pattern analysis and machine intelligence 26.9 (2004): 1124-1137.
    
    The function expects input as np.ndarray, which can be a 2D image or 3D volume
    Args:
        image: input image, can be 2D [C, H, W] or 3D [C, D, H, W].
        prob: probability in range [0, 1] and dimensions [2, H, W] or [2, D, H, W].
        lamda: weighting factor for establishing relationship between unary and pairwise terms.
        sigma: standard deviation of intensity values in neighbourhood.
        connectivity: connectivity to use, can be from [4, 8] for 2D and [6, 18, 24] for 3D
    Returns:
        np.ndarray with maxflow output
    """
    return numpymaxflowcpp.maxflow(
        image, prob, lamda, sigma, connectivity
    )

def maxflow_interactive(
    image: np.ndarray, 
    prob: np.ndarray, 
    seed: np.ndarray,
    lamda: float, 
    sigma: float, 
    connectivity: int = 0
) -> np.ndarray:
    r"""Computes Interactive Max-flow/Min-cut in numpy for 2D images and 3D volumes follwing methods described in:
    
    Boykov, Yuri, and Vladimir Kolmogorov. 
    "An experimental comparison of min-cut/max-flow algorithms for energy minimization in vision." 
    IEEE transactions on pattern analysis and machine intelligence 26.9 (2004): 1124-1137.
    
    The function expects input as np.ndarray, which can be a 2D image or 3D volume
    Args:
        image: input image, can be 2D [C, H, W] or 3D [C, D, H, W].
        prob: probability in range [0, 1] and dimensions [2, H, W] or [2, D, H, W].
        seed: seed providing user interaction input with dimensions [2, H, W] or [2, D, H, W].
        lamda: weighting factor for establishing relationship between unary and pairwise terms.
        sigma: standard deviation of intensity values in neighbourhood.
        connectivity: connectivity to use, can be from [4, 8] for 2D and [6, 18, 24] for 3D
    Returns:
        np.ndarray with maxflow output
    """
    return numpymaxflowcpp.maxflow_interactive(
        image, prob, seed, lamda, sigma, connectivity
    )


import glob
import os
import sys

from setuptools import Extension, find_packages, setup
from setuptools.command.build_ext import build_ext as _build_ext


# get numpy as dependency when it is not pre-installed
# from: https://stackoverflow.com/a/54128391/798093
class build_ext(_build_ext):
    def finalize_options(self):
        _build_ext.finalize_options(self)
        # Prevent numpy from thinking it is still in its setup process:
        import builtins

        builtins.__NUMPY_SETUP__ = False
        import numpy

        self.include_dirs.append(numpy.get_include())


def get_extensions():
    ext_dir = "numpymaxflow"
    include_dirs = [ext_dir]

    sources = glob.glob(os.path.join(ext_dir, "**", "*.cpp"), recursive=True)
    # print(sources)
    define_macros = []
    extra_link_args = []
    extra_compile_args = []

    # add any compile flags here
    # compile release
    extra_compile_args += ["-g0"]

    # use c++11 on macos
    # help from:
    # https://stackoverflow.com/a/43571056/798093
    if sys.platform == "darwin":
        # extra_compile_args += ["-stdlib=libc++"]
        extra_compile_args += ["-std=c++11"]

    if not sources:
        return []  # compile nothing

    ext_modules = [
        Extension(
            name="numpymaxflowcpp",
            sources=sources,
            include_dirs=include_dirs,
            define_macros=define_macros,
            extra_compile_args=extra_compile_args,
            extra_link_args=extra_link_args,
        )
    ]
    return ext_modules


with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt", "r") as fp:
    install_requires = fp.read().splitlines()

setup(
    name="numpymaxflow",
    version="0.0.3",
    description="numpymaxflow: Max-flow/Min-cut in Numpy for 2D images and 3D volumes",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/masadcv/numpymaxflow",
    author="Muhammad Asad",
    author_email="masadcv@gmail.com",
    license="BSD-3-Clause License",
    classifiers=[
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3",
    ],
    install_requires=install_requires,
    cmdclass={"build_ext": build_ext},  # .with_options(no_python_abi_suffix=True)},
    packages=find_packages(exclude=("data", "docs", "examples", "scripts", "tests")),
    ext_modules=get_extensions(),
    python_requires=">=3.6",
)
